<?php

function get_enviaya_accounts()
{
    $dir = WP_PLUGIN_DIR . '/enviaya-for-woocommerce/lang/';
    $request = wp_remote_get(content_url() . '/plugins/enviaya-for-woocommerce/lang/en_US.json');
    if (is_dir($dir)) {
        if ($dh = opendir($dir)) {
            while (($file = readdir($dh)) !== false) {
                if ($file === get_locale() . '.json') {
                    $request = wp_remote_get(content_url() . '/plugins/enviaya-for-woocommerce/lang/' . get_locale() . '.json');
                }
            }
            closedir($dh);
        }
    }

    global $woocommerce;

    if (isset($_POST['api_key'])) {
        $response = EYHelper::libAPI()->get_accounts($_POST['api_key']);

        if (!isset($response->errors) && isset($response->enviaya_accounts)) {
            $acc_array = $response->enviaya_accounts;
        }

        if (empty($acc_array)) {
            //echo ("<option value='none'>" . /*EYHelper::i18n('')*/ . "</option>");
            echo ("<option value='none'></option>");
        } else {
            foreach ($acc_array as $acc) {
                if ($acc->status == 'active') {
                    echo ('<option value="' . $acc->account . '">' . $acc->alias . ' (' . $acc->account .  ')</option>');
                }
            }
        }
    }
    wp_die();
}

add_action('wp_ajax_enviaya_ajax_get_accounts', 'get_enviaya_accounts');

function get_enviaya_origin_address()
{
    
    if (isset($_POST['api_key'])) {

        $props = [
            'api_key' => $_POST['api_key'],
            'param' => '&get_destinations=false',
        ];
        
        if(isset($_POST['full_name'])){
            $props['param'].='&full_name='.$_POST['full_name'];
        }else{
            $props['param'].='&per_page=50';
        }
       
        $response = EYHelper::libAPI()->directions($props);
        
        $directions = array();
        if (isset($response->directions) && !isset($response->errors)) {
            foreach ($response->directions as $address) {

                $direction = json_decode(json_encode($address), true);
                $direction['id'] = str_replace('"', '||', json_encode($address));
                $direction['text'] = $address->full_name;
                $directions[] = $direction;
                
            }
        } 

      
        
        wp_send_json(array('directions' => $directions));

       

    }
  
}

add_action('wp_ajax_enviaya_ajax_get_origin_address', 'get_enviaya_origin_address');

function downloadApiLogs()
{
    $dir = WP_PLUGIN_DIR . '/enviaya-for-woocommerce/lang/';
    $request = wp_remote_get(content_url() . '/plugins/enviaya-for-woocommerce/lang/en_US.json');
    if (is_dir($dir)) {
        if ($dh = opendir($dir)) {
            while (($file = readdir($dh)) !== false) {
                if ($file === get_locale() . '.json') {
                    $request = wp_remote_get(content_url() . '/plugins/enviaya-for-woocommerce/lang/' . get_locale() . '.json');
                }
            }
            closedir($dh);
        }
    }

    if (!class_exists('ZipArchive')) {
        die(EYHelper::i18n('no_zip_extenstion'));
    }

    $date = new \DateTime();
    $fileName = 'API_logs_' . $date->format('H_i_s d-m-Y') . '.zip';
    $filePath = ABSPATH . "wp-content/" . $fileName;
    $zip = new ZipArchive();
    $zip->open($fileName, ZipArchive::CREATE | ZipArchive::OVERWRITE);
    $zipOptions = [
        'remove_all_path' => TRUE,
    ];

    $zip->addGlob(ENVIAYA_API_LOGS_FOLDER . "/*", 0, $zipOptions);


    $zip->close();
    header("Expires: 0");
    header("Cache-Control: no-cache, no-store, must-revalidate");
    header('Cache-Control: pre-check=0, post-check=0, max-age=0', false);
    header("Pragma: no-cache");
    header("Content-type: zip");
    header("Content-Disposition:attachment; filename={$fileName}");
    header("Content-Type: application/force-download");
    readfile($fileName);
    wp_die();
}
add_action('wp_ajax_ey_download_api_logs', 'downloadApiLogs');

function deleteApiLogs()
{
    $files = glob(ENVIAYA_API_LOGS_FOLDER . '/*');
    foreach ($files as $file) {
        if (is_file($file))
            try {
                unlink($file);
            } catch (\Exeption $e) {
                echo $e;
            }
    }
    wp_die();
}
add_action('wp_ajax_ey_delete_api_logs', 'deleteApiLogs');


add_action('wp_ajax_enviaya_get_news_accounts', 'getNewsAccountEnviaYa');
function getNewsAccountEnviaYa()
{   
    $response = EYHelper::libAPI()->get_accounts(null);
    echo json_encode($response);
    wp_die();
}

add_action('wp_ajax_enviaya_get_news_directions', 'getNewsDirectionsEnviaYa');
function getNewsDirectionsEnviaYa()
{   
    $props = [
        'param' => '&get_origins=t',
    ];
    $response = EYHelper::libAPI()->directions($props);
    echo json_encode($response);
    wp_die();
}

add_action('wp_ajax_enviaya_search_carrier_service', 'get_enviaya_carrier_service');
function get_enviaya_carrier_service()
{
    $settings = EYHelper::settings();
    $services = json_decode($settings["eya_available_services"]);

    echo json_encode(EYHelper::getSearchServiceSelected($_POST["carrier"], $services));
    wp_die();
}

add_action( "wp_ajax_enviaya_setting_bulk_carrier", "set_setting_bulk_carrier");
function set_setting_bulk_carrier()
{
    EYHelper::updateSetting("eya_option_service", $_POST["optionService"]);
    EYHelper::updateSetting("eya_new_option_carrier", $_POST["newOptionCarrier"]);
    EYHelper::updateSetting("eya_new_option_service", $_POST["newOptionService"]);
    EYHelper::updateSetting("eya_setting_secondary_service", $_POST["settingSecondaryService"]);
    EYHelper::updateSetting("eya_new_option_secondary_carrier", $_POST["newOptionSecondaryCarrier"]);
    EYHelper::updateSetting("eya_new_option_secondary_service", $_POST["newOptionSecondaryService"]);

    echo json_encode(["success" => true]);
    wp_die();
}
