<?php

include_once(ABSPATH . 'wp-admin/includes/plugin.php');

class EYHelper
{

    private static $jsonTranslation;

    public static function getJsonTranslation()
    {
        if (self::$jsonTranslation === null) {
            self::$jsonTranslation = json_decode(file_get_contents(__DIR__ . '/translation.json'), true);
        }

        return self::$jsonTranslation;
    }

    public static function i18n($key)
    {
        $array = self::getJsonTranslation();

        /* Make exception whenever we haven't support for other lenguages with -----_Portugues */
        if ($key == null) {
            return $array;
        }

        $language    = strpos(get_user_locale(), "_");
        $languageKey = substr(get_user_locale(), 0, $language);
        
        if(!isset($array[$key][$languageKey]))
        {
            error_log("the key $key is not enabled inside translation.js file");
        }       

        return $array[$key][$languageKey];
    }

    public static function makeDestiny($shipping)
    {
        $settings = self::settings();

        return [
            'full_name'    => $shipping['shipping']['first_name'] . ' ' . $shipping['shipping']['last_name'],
            'company'      => $shipping['shipping']['company'],
            'country_code' => $shipping['shipping']['country'],
            'postal_code'  => $shipping['shipping']['postcode'],
            'direction_1'  => $shipping['shipping']['address_1'],
            'district'     => $shipping['shipping']['address_2'],
            'phone'        => $shipping['billing']['phone'],
            'state_code'   => $shipping['shipping']['state'],
            'city'         => $shipping['shipping']['city'],
            'email'        => $settings['send_shipment_notice'] == '0' ? null: $shipping['billing']['email'],
        ];
    }

    /* Used by callback, do not remove! */
    private static function upperize($el)
    {
        return str_replace('..', '.', mb_convert_case($el, MB_CASE_TITLE, "UTF-8"));
    }

    public static function is_woo()
    {
        $url = $_SERVER['REQUEST_URI'];
        $page_id = url_to_postid($url);

        if ((is_cart() || get_option('woocommerce_cart_page_id') == $page_id) || (is_checkout() || get_option('woocommerce_checkout_page_id') == $page_id) || $url == '/?wc-ajax=update_order_review' || is_admin()) {
            return true;
        }

        return false;
    }

    public static function formatDate(\Datetime $date, $format, $locale = 'en')
    {
        if (!class_exists('IntlDateFormatter')) {
            return ucwords(date_i18n($format, $date->getTimestamp()));
        }

        $intlFormats = [
            'l, M. j' => 'EEEE, LLL. dd'
        ];

        $formatter = new \IntlDateFormatter($locale, \IntlDateFormatter::LONG, \IntlDateFormatter::LONG);
        $formatter->setPattern($intlFormats[$format]);

        return implode(' ', array_map('EYHelper::upperize', explode(' ', $formatter->format($date))));
    }

    public static function buildPackage($order_id)
    {
        $order   = wc_get_order($order_id);
        $items   = $order->get_items();
        $package = array();

        foreach ($items as $item) {
            
            $thisParcel = null;
            
            $_Q = $item['quantity'];
            $_qty = (float)$_Q;

            $data = $item->get_data();
            $product = wc_get_product($data['variation_id'] > 0 ? $data['variation_id']: $data['product_id']);
               
            $_weight = (float)$product->get_weight();

            $tmpLength = $product->get_length() ? (float)$product->get_length() : null;
            $tmpHeight = $product->get_height() ? (float)$product->get_height() : null;
            $tmpWidth = $product->get_width() ? (float)$product->get_width() : null;

            $weightUnit = get_option('woocommerce_weight_unit');
            $dimensionUnit = get_option('woocommerce_dimension_unit');

            if ($_weight && $weightUnit === 'g') {
                $_weight /= 1000;
                $weightUnit = 'kg';
            }

            if ($_weight && $weightUnit === 'oz') {
                $_weight /= 35.274;
                $weightUnit = 'kg';
            }

            $endWeight = round($_weight, 2);

            $thisParcel = array(
                'quantity'      => $_qty,
                'weight'        => $endWeight ? round($endWeight, 2) : null,
                'weight_unit'   => $weightUnit,
                'length'        => $tmpLength ? round($tmpLength, 2) : null,
                'height'        => $tmpHeight ? round($tmpHeight, 2) : null,
                'width'         => $tmpWidth ? round($tmpWidth, 2) : null,
                'dimension_unit' => $dimensionUnit,
            );

            $package[] = $thisParcel;


        }

        return $package;
    }

    public static function libAPI()
    {
        $settings = self::settings();

        return new EnviayaAPI([
            'api_key'                   => $settings['api_key'],
            'enviaya_account'           => $settings['enviaya_account'],
            'carrier_account'           => null,
            'app_id'                    => 1,
            'domain'                    => self::i18n('api_domain'),
            'timeout'                  =>  $settings['timeout']
        ]);
    }

    public static function libAPIdokan($dokan)
    {
        $settings = self::settings();

        return new EnviayaAPI([
            'api_key'                   => $dokan['api_key'],
            'enviaya_account'           => $dokan['account'],
            'carrier_account'           => null,
            'app_id'                    => 1,
            'domain'                    => self::i18n('api_domain'),
            'timeout'                   =>  $settings['timeout']
        ]);
    }

    public static function create($data)
    {
        $settings = self::settings();
        $currency = isset($settings['enable_currency_support']) &&
        $settings['enable_currency_support'] === 'yes' ? get_woocommerce_currency() : (isset($settings['default_currency']) && $settings['default_currency'] ?
            $settings['default_currency'] : get_woocommerce_currency());

        $props = [
            'rate_currency'             => $currency,
            'shipment_type'             => 'Package',
            'parcels'                   => $data['parcels'],
            'origin_country_code'       => $data['origin_country_code'],
            'origin_postal_code'        => $data['origin_postal_code'],
            'origin_state_code'         => $data['origin_state_code'],
            'destination_country_code'  => $data['destination_country_code'],
            'destination_postal_code'   => $data['destination_postal_code'],
            'destination_state_code'    => $data['destination_state_code'],
            'currency'                  => $currency,
            'order_total_amount'        => self::truncateVal($data['order_total_amount'], 0),
            'locale'                    => get_user_locale()
        ];

        return $props;
    }
    
    public static function getMetaOrderItems($order_id_current)
    {
        global $wpdb;
        
        $qry =  "SELECT * FROM {$wpdb->prefix}woocommerce_order_items WHERE order_id = {$order_id_current} AND order_item_type = 'shipping'";

        $states = $wpdb->get_results($qry);
        
        $qry = "SELECT * FROM {$wpdb->prefix}woocommerce_order_itemmeta WHERE order_item_id = {$states[0]->order_item_id}";
        
        return $wpdb->get_results($qry);
    }
    
    public static function getCarrierData($states)
    {
        $carrier_service_code = '';
        $carrier_name         = '';

        foreach ($states as $value) {
            if ($value->meta_key == 'carrier_name') {
                $carrier_name = $value->meta_value;
            }

            if ($value->meta_key == 'carrier_service_code') {
                $carrier_service_code = $value->meta_value;
            }
        }
        
        return [
                "carrier_service_code" => $carrier_service_code, 
                "carrier_name"         => $carrier_name
                ];
    }
    
    public static function getDBOrderDataRates($order_id_current, $carrier_service_code, $carrier_name)
    {
        global $wpdb;
        
        $qry = "SELECT * FROM {$wpdb->prefix}" . PREFIX . "_rates WHERE
                order_id = {$order_id_current} AND carrier_service_code = '{$carrier_service_code}' AND
                carrier = '{$carrier_name}';";
                
        return $wpdb->get_results($qry);
    }
    
    public static function validateOrderCountForShipping($data, $order_rate, $order_id_current)
    {
        if (count($order_rate)) 
        {
            return true;
        }       

        return false;
    }
    
    public static function reloadRatingOptions($data)
    {
        global $wpdb;
        
        $settings = EYHelper::settings();
        $order    = $data['get_post'];
        $orderw   = $data['wc_get_order'];

        $origin_address = $settings['origin_address'];
        $parcels        = EYHelper::buildPackage($orderw->get_id());

        $currency = isset($settings['enable_currency_support']) &&
            $settings['enable_currency_support'] === 'yes' ? get_woocommerce_currency() : (isset($settings['default_currency']) && $settings['default_currency'] ?
                $settings['default_currency'] : get_woocommerce_currency());


        $props = [
            'rate_currency'             => $currency,
            'shipment_type'             => 'Package',
            'parcels'                   => $parcels,
            'origin_country_code'       => isset($origin_address->country_code) ? $origin_address->country_code : null,
            'origin_postal_code'        => isset($origin_address->postal_code) ? $origin_address->postal_code : null,
            'origin_state_code'         => isset($origin_address->state_code) ? $origin_address->state_code : null,
            'destination_country_code'  => $orderw->get_shipping_country(),
            'destination_postal_code'   => $orderw->get_shipping_postcode(),
            'destination_state_code'    => $orderw->get_shipping_state(),
            'currency'                  => $currency,
            'locale'                    => get_user_locale(),
        ];

        $response = EYHelper::libAPI()->calculate($props)['response'];

        if ($response) {
            $rate_list = array();
            foreach ($response as $key => $value) {
                if ($key === 'store_pickup') {
                    $rate_list[] = $value;
                }

                if ($key !== 'warning' && $key !== 'errors' && $key = 'store_pickup') {
                    foreach ($value as $resp) {
                        $rate_list[] = $resp;
                    }
                }
            }

            foreach ($rate_list as $key => $resp) {
                if (isset($resp->rate_id)) {
                    $wpdb->get_results("INSERT INTO {$wpdb->prefix}" . PREFIX . "_rates (order_id, rate_id,
                        shipment_id, carrier, carrier_service_name, carrier_service_code, estimated_delivery, currency,
                        carrier_logo_url, total_amount, net_total_amount, dynamic_service_name, label_rates) VALUES
                        (" . $orderw->get_id() . ", " . $resp->rate_id . ", " . $resp->shipment_id . ", '" . $resp->carrier . "',
                        '" . $resp->carrier_service_name . "', '" . $resp->carrier_service_code . "',
                        '" . $resp->estimated_delivery . "', '" . $resp->currency . "', '" . $resp->carrier_logo_url . "',
                        '" . $resp->total_amount . "', '" . $resp->net_total_amount . "', '" . $resp->dynamic_service_name . "',
                        '');");                    
                }
            }
            
            return true;
        }
        
        update_post_meta($orderw->get_id(), "eya_message_auto_shipment", EYHelper::i18n('automatic_booking_error_message_default'));
        update_post_meta($orderw->get_id(), "eya_status_shipment_booking", "fail");
        
        return false;
    }
    
    
    
    public static function getDBShippingData($order_id_current)
    {
        global $wpdb;
        
        //Validate if shipment was building        
        $qry = "SELECT * FROM {$wpdb->prefix}" . PREFIX . "_shipment WHERE 
            shipment_status='Envío generado' AND  order_id = " . $order_id_current ;      
        
        return $wpdb->get_results($qry);                      
    }
    
    public static function validateShippingBuilding($order_id_current, $shipment_status)
    {
        if(!count($shipment_status))
        {
            return false;
        }
        
        $error_message_existing = str_replace(
            ['{#carrier_tracking_number#}', '{#carrier#}', '{#service_name#}'],
            [ $shipment_status[0]->carrier_shipment_number, $shipment_status[0]->carrier, $shipment_status[0]->carrier_service_name ],
            EYHelper::i18n('automatic_booking_error_message_existing_shipment')
        );

        update_post_meta($order_id_current, "eya_message_auto_shipment", $error_message_existing);
        update_post_meta($order_id_current, "eya_status_shipment_booking", "fail");

        return true;           
    }
    
    public static function validateExistsCarrierInOptionsFaster($carrier_name, $faster_carriers)
    {
        error_log("the option selected is not same that options in DB, see the qry");
            
        if(!in_array( $carrier_name, $faster_carriers ))
        {
            return true;
        }

        return false;        
    }
    
    public static function validateCarriersAndDeliveryDate($order_id_current, $order_rate, $carrier_name)
    {
        $faster_carriers = ["iVoy", "Minutos"];
        
        if(!count($order_rate))
        {
            if(!self::validateExistsCarrierInOptionsFaster($carrier_name, $faster_carriers))
            {
                update_post_meta($order_id_current, "eya_message_auto_shipment", EYHelper::i18n('automatic_booking_error_message_default'));
                update_post_meta($order_id_current, "eya_status_shipment_booking", "fail");

                return false;
            } 
            
            return true;
        }
        
        //Validate carriers and delivery date
        $date_future     = strtotime(date("Y-m-d")."+ 1 days");
        $date_shipment   = strtotime($order_rate[0]->estimated_delivery);
        
        if((int)$date_shipment < (int)$date_future && in_array( $carrier_name, $faster_carriers ))
        {
            update_post_meta($order_id_current, "eya_message_auto_shipment", EYHelper::i18n("automatic_booking_error_message_shipment_date_today"));
            update_post_meta($order_id_current, "eya_status_shipment_booking", "fail");

            return false;
        }//end 
        
        return true;
    }
    
    public static function validateAutomaticShipmentBooking($data, $order_id_current, $carrier_service_code, $carrier_name)
    {
        /* if(isset($data['auto_shipment_booking']))
        {         */
            // automatic_shipment_booking
            $order_rate = self::getDBOrderDataRates($order_id_current, $carrier_service_code, $carrier_name);
            
            if(!self::validateOrderCountForShipping($data, $order_rate, $order_id_current))
            {
                //Again show options for raiting
                if(!self::reloadRatingOptions($data))
                {
                    return false;
                }
                
                //Get new options rates
                $order_rate = self::getDBOrderDataRates($order_id_current, $carrier_service_code, $carrier_name);                
            }

            if(self::validateShippingBuilding($order_id_current, self::getDBShippingData($order_id_current)))
            {
                return false;
            }

            if(!self::validateCarriersAndDeliveryDate($order_id_current, $order_rate, $carrier_name))
            {
                return false;
            }
        // }
        
        return true;
    }

    public function isEnabledAutomaticShipment($settings)
    {

    }

    public static function create_shipment($data)
    {
        global $wpdb;
        
        if(!isset($data['get_post']))
        {
            return false;
        }
        
        $settings             = self::settings();
        // $order_id_current     = isset($data['get_post']->ID)? $data['get_post']->ID : $data['get_post'];          
        $order_id_current     = $data['wc_get_order']->get_id();
        $data_carrier         = self::getCarrierData(self::getMetaOrderItems($order_id_current));
        $carrier_service_code = $data_carrier['carrier_service_code'];
        $carrier_name         = $data_carrier['carrier_name'];

        $dataAuto = self::validateAutomaticShipmentBooking($data, $order_id_current, $carrier_service_code, $carrier_name);        

        if(!$dataAuto){
            return false;
        }

        $order_id = $order_id_current;

        if (isset($settings['origin_address']->country_code) && isset($settings['origin_address']->postal_code))
        {

            $country_code = $settings['origin_address']->country_code;
            $postal_code  = $settings['origin_address']->postal_code;
            $full_name    = $settings['origin_address']->full_name;
            $company      = $settings['origin_address']->company;
            $direction_1  = $settings['origin_address']->direction_1;
            $city         = $settings['origin_address']->city;
            $phone        = $settings['origin_address']->phone;
            $state_code   = $settings['origin_address']->state_code;
            $neighborhood = $settings['origin_address']->neighborhood;
            $district     = $settings['origin_address']->district;
            $email        = $settings['origin_address']->email;
            
            
            if(isset($data['dokan']) && !empty($data["dokan"])){
                $origin  = json_decode(base64_decode(get_user_meta($data['vendor_id'], 'dokan_sender_address', true)));
                $country = get_option('woocommerce_default_country');
                $arr     = explode(":", $country);
                $country = $arr[0];

                $country_code = isset($origin->country_code) ? $origin->country_code: $country;
                $postal_code  = isset($origin->postal_code) ? $origin->postal_code  : null;
                $state_code   = isset($origin->state_code) ? $origin->state_code    : null;
                $full_name    = isset($origin->full_name) ? $origin->full_name      : null;
                $company      = isset($origin->company) ? $origin->company          : null;
                $direction_1  = isset($origin->direction_1) ? $origin->direction_1  : null;
                $city         = isset($origin->city) ? $origin->city                : null;
                $phone        = isset($origin->phone) ? $origin->phone              : null;
                $neighborhood = isset($origin->neighborhood) ? $origin->neighborhood: null;
                $district     = isset($origin->district) ? $origin->district        : null;
                $email        = isset($origin->email) ? $origin->email              : null;

            }

            $order    = wc_get_order( $order_id );
            $shipping = $order->get_data();
            $_destiny = EYHelper::makeDestiny($shipping);
            $parcels  = EYHelper::buildPackage($order_id);

            if (isset($data['rate_id'])) {
                $states3 = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}" . PREFIX . "_rates WHERE rate_id = {$data['rate_id']}");
                $carrier_service_code = $states3[0]->carrier_service_code;
                $carrier_name         = $states3[0]->carrier;
            }
        }
        
        $props = [
            'rate_id'                   => isset($data['rate_id']) ? $data['rate_id'] : null,
            'shipment_id'               => isset($data['shipment_id']) ? $data['shipment_id'] : null,
            'rate_currency'             => $settings['enable_currency_support'] === 'yes' ? get_woocommerce_currency() : $settings['default_currency'],
            'carrier'                   => $carrier_name,
            'carrier_service_code'      => $carrier_service_code,
            'origin_full_name'          => $full_name,
            'origin_company'            => $company,
            'origin_country_code'       => $country_code,
            'origin_postal_code'        => $postal_code,
            'origin_direction_1'        => $direction_1,
            'origin_city'               => $city,
            'origin_phone'              => $phone,
            'origin_state_code'         => $state_code,
            'origin_neighborhood'       => $neighborhood,
            'origin_district'           => $district,
            'origin_email'              => $email,
            'destination_full_name'     => $_destiny['full_name'],
            'destination_company'       => $_destiny['company'],
            'destination_country_code'  => $_destiny['country_code'],
            'destination_postal_code'   => $_destiny['postal_code'],
            'destination_direction_1'   => $_destiny['direction_1'],
            'destination_district'      => $_destiny['district'],
            'destination_phone'         => $_destiny['phone'],
            'destination_state_code'    => $_destiny['state_code'],
            'destination_city'          => $_destiny['city'],
            'destination_email'         => $_destiny['email'],
            'parcels'                   => $parcels,
            'content'                   => 'Varios',
            'label_format'              => 'Letter',
            'shipment_type'             => 'Package',
            'order_total_amount'        => self::truncateVal(($data["wc_get_order"]->get_total() - $data["wc_get_order"]->get_total_shipping() - $data["wc_get_order"]->get_shipping_tax()), 0),
            'currency'                  => $data["wc_get_order"]->get_currency(),
            'shop_order_id'             => $order_id,
            'locale'                    => get_user_locale()
        ];        

        return $props;
    }

    public static function settings()
    {
        $settings = get_option('woocommerce_enviaya_settings');

        $api_key_production = isset($settings['api_key_production']) ? $settings['api_key_production'] : null;
        $api_key_test       = isset($settings['api_key_test']) ? $settings['api_key_test'] : null;
        $enabled_test_mode  = isset($settings['enabled_test_mode']) ? $settings['enabled_test_mode'] : 'no';
        $api_key            = $enabled_test_mode == 'yes' ? $api_key_test : $api_key_production;

        return [
            'api_key'                           => $api_key,
            'enviaya_account'                   => isset($settings['enviaya_account']) ? $settings['enviaya_account'] : '',
            'enable_rating'                     => isset($settings['enable_rating']) ? $settings['enable_rating'] : '1',
            'shipping_service_design_advanced'  => isset($settings['shipping_service_design_advanced']) ? $settings['shipping_service_design_advanced'] : '0',
            'shipping_carrier_name'             => isset($settings['shipping_carrier_name']) ? $settings['shipping_carrier_name'] : '0',
            'shipping_delivery_time'            => isset($settings['shipping_delivery_time']) ? $settings['shipping_delivery_time'] : '0',
            'default_or_advanced_design'        => isset($settings['default_or_advanced_design']) ? $settings['default_or_advanced_design'] : '0',
            'display_carrier_logo'              => isset($settings['display_carrier_logo']) ? $settings['display_carrier_logo'] : '0',
            'shipping_service_design'           => isset($settings['shipping_service_design']) ? $settings['shipping_service_design'] : '1',
            'group_by_carrier'                  => isset($settings['group_by_carrier']) ? $settings['group_by_carrier'] : '0',
            'as_defined_price'                  => isset($settings['as_defined_price']) ? $settings['as_defined_price'] : '0',
            'enable_contingency_shipping'       => isset($settings['enable_contingency_shipping']) ? $settings['enable_contingency_shipping'] : '1',
            'enable_standard_flat_rate'         => isset($settings['enable_standard_flat_rate']) ? $settings['enable_standard_flat_rate'] : 'yes',
            'standard_flat_rate'                => isset($settings['standard_flat_rate']) ? $settings['standard_flat_rate'] : '100',
            'enable_express_flat_rate'          => isset($settings['enable_express_flat_rate']) ? $settings['enable_express_flat_rate'] : 'yes',
            'express_flat_rate'                 => isset($settings['express_flat_rate']) ? $settings['express_flat_rate'] : '150',
            'default_currency'                  => isset($settings['default_currency']) ? $settings['default_currency'] : get_woocommerce_currency(),
            'enable_currency_support'           => isset($settings['enable_currency_support']) ? $settings['enable_currency_support'] : 'no',
            'rate_on_add_to_cart'               => isset($settings['rate_on_add_to_cart']) ? $settings['rate_on_add_to_cart'] : '0',
            'send_shipment_notice'              => isset($settings['send_shipment_notice']) ? $settings['send_shipment_notice'] : '0',
            'availability'                      => isset($settings['availability']) ? $settings['availability'] : 'all',
            'countries'                         => isset($settings['countries']) ? $settings['countries'] : '',
            'doken_integration'                 => isset($settings['doken_integration']) ? $settings['doken_integration'] : 'yes',
            'timeout'                           => isset($settings['timeout']) ? $settings['timeout'] : '15',
            'enable_api_logging'                => isset($settings['enable_api_logging']) ? $settings['enable_api_logging'] : 'no',
            'total_remove'                      => isset($settings['total_remove']) ? $settings['total_remove'] : 'no',
            'excluded_zones_data'               => isset($settings['excluded_zones_data']) ? $settings['excluded_zones_data'] : [],
            'origin_address'                    => isset($settings['origin_address']) ? json_decode(str_replace('||', '"',  $settings['origin_address'])) : null,
            'enable_estimated_shipping'         => isset($settings['enable_estimated_shipping']) ? $settings['enable_estimated_shipping'] : '0',
            'account_id'                        => isset($settings['account_id']) ? $settings['account_id'] : null,
            'title_rating_configuration'        => isset($settings['title_rating_configuration_'.get_locale()]) ? $settings['title_rating_configuration_'.get_locale()] : null,
            'automatic_shipment_booking'        => isset($settings['automatic_shipment_booking']) ? $settings['automatic_shipment_booking'] : 'no',
            /* Notices message */
            'eya_notice_show'                   => isset($settings['eya_notice_show']) ? $settings['eya_notice_show'] : 'no',
            'eya_notice_message'                => isset($settings['eya_notice_message']) ? $settings['eya_notice_message'] : '',
            'eya_notice_class'                  => isset($settings['eya_notice_class']) ? $settings['eya_notice_class'] : 'notice-info',
            'eya_public_pdf_shipment'           => isset($settings['eya_public_pdf_shipment']) ? $settings['eya_public_pdf_shipment'] : '',
            /* Bulk carrier setting */            
            'eya_available_services'            => isset($settings['eya_available_services']) ? $settings['eya_available_services'] : '',
            'eya_option_service'                => isset($settings['eya_option_service']) ? $settings['eya_option_service'] : 'default',
            'eya_new_option_carrier'            => isset($settings['eya_new_option_carrier']) ? $settings['eya_new_option_carrier'] : '',
            'eya_new_option_service'            => isset($settings['eya_new_option_service']) ? $settings['eya_new_option_service'] : '',
            'eya_setting_secondary_service'     => isset($settings['eya_setting_secondary_service']) ? $settings['eya_setting_secondary_service'] : 'false',
            'eya_new_option_secondary_carrier'  => isset($settings['eya_new_option_secondary_carrier']) ? $settings['eya_new_option_secondary_carrier'] : '',
            'eya_new_option_secondary_service'  => isset($settings['eya_new_option_secondary_service']) ? $settings['eya_new_option_secondary_service'] : '',
        ];
    }

    public static function updateSetting($new_setting, $value_setting)
    {
        $config_setting               = get_option('woocommerce_enviaya_settings');
        $config_setting[$new_setting] = $value_setting;

        update_option('woocommerce_enviaya_settings', $config_setting);
    }

    public static function getReplacingKeyi18n($needle, $key, $key_parent)
    {
        return str_replace($needle, self::i18n($key), self::i18n($key_parent));
    }

    public static function getAccountId($cur_account)
    {
        $response = self::libAPI()->get_accounts(null);

        if (!isset($response->errors) && isset($response->enviaya_accounts)) {
            $acc_array = $response->enviaya_accounts;
        }

        if (isset($acc_array)) {
            foreach ($acc_array as $acc) {
                if ($acc->status == 'active' && $acc->account === $cur_account) {
                    return $acc->id;
                    break;
                }
            }
        }

        return null;
    }

    /**
     * @example truncate(-1.49999, 2); // returns -1.49
     * @example truncate(.49999, 3); // returns 0.499
     * @param float $val
     * @param int f
     * @return float
     */
    public static function truncateVal($val, $f = 0)
    {
        if(($p = strpos($val, '.')) !== false) {
            $val = floatval(substr($val, 0, $p + 1 + $f));
        }
        return $val;
    }
    
    public static function getOptionsAvailableServices($carrier = false, $service_selected = false)
    {
        $response = self::libAPI()->get_available_service(null);
        $service_selected = ($service_selected) ? $service_selected : "";
        $carrier = ($carrier) ? $carrier : "UPS";

        EYHelper::updateSetting("eya_available_services", json_encode($response));
        $options_service_first = "";
        foreach(self::getSearchServiceSelected($carrier, $response) as $key => $value)
        {
            $options_service_first .= '<option value="'.$key.'" '.($key === $service_selected? 'selected' : '').'>'.$value.'</option>';
        }

        return $options_service_first;
    }

    public static function getSearchServiceSelected($needle_service_name, $response)
    {
        $needle_service_name = (strcmp($needle_service_name, "FedEx") == 0) ? $needle_service_name."®" : $needle_service_name;
        $array_service_first = array();
        foreach ($response->services as $carrier_key => $service_carrier) 
        {   
            foreach($service_carrier as $key_service => $service_name )
            {
                if($key_service == $needle_service_name)
                {
                    foreach ($service_name as $service_key => $service_value) 
                    {
                        foreach ($service_value as $key => $value) 
                        {
                            // $options_service_first .= '<option value="'.$key.'">'.$value.'</option>';
                            $array_service_first[$key] = $value;
                        }
                    }
                }                
                break;                
            }
        }       

        return $array_service_first;
    }

    public static function getOptionsAvailableCarriers($carrier_selected = false)
    {
        $response         = self::libAPI()->get_available_carrier(null);
        $carrier_selected = ($carrier_selected) ? $carrier_selected : "";

        $options_carriers = "";        
        foreach ($response->carriers as $carrier) {
            foreach ($carrier as $key => $value) {
                $options_carriers .= '<option value="'.$value.'" '.($value === $carrier_selected? 'selected' : '').'  >'.$value.'</option>';                
            }            
        }

        return $options_carriers;
    }

}
