<?php
/**
 * @package EnvíaYa
 * @version 1.14.5
 * 
 * Plugin Name: EnvíaYa for WooCommerce
 * Description: An powerful plugin to rate multi-carrier shipment services and est. delivery dates during checkout, create shipment labels with one click and track.
 * Author: Envia Ya S.A. de C.V
 * Version: 1.14.5
 * Author URI: http://enviaya.com.mx/
 * Text Domain: enviaya
 */

define('ENVIAYA_VERSION', '1.14.5');

if ( ! defined( 'WPINC' ) ) {
    die;
}

if (!defined('ENVIAYA_PLUGIN')) {
    define('ENVIAYA_PLUGIN', 'enviaya-for-woocommerce.php');
}

add_filter( 'wc_get_template', function ( $file, $name ) {
    if ( $name === 'order/order-details.php' ) {
        $file = __DIR__ . '/public/templates/order-details.php';
    }
    return $file;
}, 10, 2);

add_filter('wc_get_template', function ($file, $name) {
    if ($name === 'cart/cart-shipping.php') {
        $file = __DIR__ . '/public/templates/cart-shipping.php';
    }
    return $file;
}, 10, 2);

add_filter('plugin_row_meta', 'wk_plugin_row_meta', 10, 2);
function wk_plugin_row_meta($links, $file)
{
    if (plugin_basename(__FILE__) == $file) {
        $row_meta = array(
            'author_uri'        => '<a href="' . esc_url('https://enviaya.com.mx/') . ' "target="_blank" >' . esc_html__(EYHelper::getReplacingKeyi18n('{#brand_name#}', 'brand_name', 'author_uri'), 'domain') . '</a>',
            'documentation'     => '<a href="' . esc_url('https://soporte.enviaya.com.mx/portal/es/kb/articles/instalacion-plugin-woocommerce#Paso_3_Configura_tu_plugin_en_tu_tienda_WooCommerce') . ' "target="_blank" >' . esc_html__(EYHelper::i18n('documentation'), 'domain') . '</a>',
            'premium_support'   => '<a href="' . esc_url('https://soporte.enviaya.com.mx/portal/es/newticket') . ' "target="_blank" >' . esc_html__(EYHelper::i18n('premium_support'), 'domain') . '</a>'
        );

        return array_merge($links, $row_meta);
    }
    return (array) $links;
}

add_action('admin_menu', 'register_my_custom_submenu_page', 60);
function register_my_custom_submenu_page()
{
    $brand_name = EYHelper::i18n('brand_name');

    add_submenu_page('woocommerce', $brand_name, $brand_name, 'manage_options', 'admin.php?page=wc-settings&amp;tab=shipping&amp;section=enviaya');
    
    enviaya_front_ajax();
}

if (!function_exists('is_plugin_active')) {
    include_once(ABSPATH . 'wp-admin/includes/plugin.php');
}

if (is_plugin_active('woocommerce/woocommerce.php'))
{
    // check if accessed directly
    if (!defined('ABSPATH')) {
        exit;
    }

    // check if WooCommerce is active
    if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
        exit;
    }

    // constant this file
    if (!defined('ENVIAYA_FILE')) {
        define('ENVIAYA_FILE', __FILE__);
    }

    if (!defined('ENVIAYA_API_LOGS_FOLDER')) {
        define('ENVIAYA_API_LOGS_FOLDER', WP_PLUGIN_DIR . '/enviaya-for-woocommerce/api-logs');
        if (!is_dir(ENVIAYA_API_LOGS_FOLDER)) {
            mkdir(ENVIAYA_API_LOGS_FOLDER);
        }
    }

    require_once __DIR__ . "/includes/enviaya.php";
    require_once __DIR__ . "/admin/helpers/hooks.php";
    require_once __DIR__ . "/admin/enviaya-for-woocommerce-admin.php";
    require_once __DIR__ . "/admin/ajax/ajax.php";
    require_once __DIR__ . "/includes/ajax.php";
    require_once( plugin_dir_path( __FILE__ ) . '/admin/lib/action-scheduler/action-scheduler.php' );

    // admin-side css
    function enviaya_admin_styles()
    {
        
        wp_register_style('ey-css', plugins_url('admin/css/enviaya.css', ENVIAYA_FILE),array(),null);
        wp_enqueue_style( 'ey-css' );

        wp_register_style('ey-select2-css',plugin_dir_url( __FILE__ ) . 'admin/lib/select2/css/select2.min.css',array(),null);
        wp_enqueue_style( 'ey-select2-css' );
    

    }
    add_action('admin_head', 'enviaya_admin_styles');

    // admin-side ajax
    function enviaya_admin_js()
    {
        $lang = EYHelper::i18n(null);
        $language    = strpos(get_user_locale(), "_");
        $languageKey = substr(get_user_locale(), 0, $language);

        wp_register_script('ey-admin-ajax', plugins_url('admin/js/ajax.js', ENVIAYA_FILE), array ( 'jquery', 'wp-i18n' ), null, true);
        wp_localize_script('ey-admin-ajax', 'EnviayaAjax', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'locale' => $languageKey,
            'lang' => $lang,
        ));
        


        global $wp_scripts;
        global $enqueued_scripts;
       
        $enqueued_scripts = array();
       
        foreach( $wp_scripts->queue as $handle ) {

            if (strpos($handle, 'select2') !== false) {
                wp_dequeue_script( $handle );
                wp_deregister_script( $handle2 );
            }else{
                $enqueued_scripts[] = $wp_scripts->registered[$handle]->src;
            }

        }
        
        wp_register_script( 'ey-select2-js', plugin_dir_url( __FILE__ ) . 'admin/lib/select2/js/select2.min.js', array(), null, true );
        wp_enqueue_script('ey-select2-js');
        
        wp_enqueue_script('ey-admin-ajax');
        wp_set_script_translations('instant-search', 'instantsearch', plugins_url('languages', __FILE__));
        
    }
    add_action('admin_enqueue_scripts', 'enviaya_admin_js',PHP_INT_MAX);

    // front-side ajax
    function enviaya_front_ajax()
    {
        wp_register_script('ey-ajax', plugins_url('public/js/ajax.js', ENVIAYA_FILE), array('jquery') );
        wp_localize_script('ey-ajax', 'EnviayaAjax', array('ajaxurl' => admin_url('admin-ajax.php')));
        wp_enqueue_script('ey-ajax');
    }
    add_action('wp_enqueue_scripts', 'enviaya_front_ajax');

    //front-side css
    function enviaya_front_css()
    {
        wp_register_style('ey-css-public', plugins_url('public/css/enviaya.css', ENVIAYA_FILE));
        wp_enqueue_style('ey-css-public', plugins_url('public/css/enviaya.css', ENVIAYA_FILE));
    }
    add_action('wp_enqueue_scripts', 'enviaya_front_css');   

    // hooks
    add_filter('plugin_action_links_' . plugin_basename(ENVIAYA_FILE), ['Enviaya', 'plugin_settings_link']);

    //intial shipping methods
    add_filter('woocommerce_shipping_methods',  ['Enviaya', 'shipping_methods']);

    add_filter('allowed_http_origins', 'add_allowed_enviaya_origins');

    function add_allowed_enviaya_origins($origins)
    {
        $origins[] = get_site_url();
        return $origins;
    }


    register_activation_hook( __FILE__, array( 'Enviaya', 'activate' ) );
    register_deactivation_hook( __FILE__, array( 'Enviaya', 'deactivation' ) );
    

    function my_enviaya_endpoints()
    {
        add_rewrite_endpoint('webhook', EP_ROOT | EP_PAGES);
    }

    add_action('init', 'my_enviaya_endpoints');

    function my_enviaya_func($args)
    {
        global $wpdb;

        if (isset($args['status'])) {
            if (strtoupper($args['status']) == "SUCCESS") {
                switch ($args['action']) {
                    case 'status_update':
                        try {
                            $wpdb->get_results("UPDATE {$wpdb->prefix}" . PREFIX . "_shipment SET
                                status = '{$args['shipment']['status']}',
                                carrier_shipment_number = '{$args['shipment']['carrier_shipment_number']}',
                                shipment_id = '{$args['shipment']['shipment_id']}',
                                event_code = '{$args['shipment']['event_code']}',
                                event_description = '{$args['shipment']['event_description']}',
                                event = '{$args['shipment']['event']}',
                                status_code = '{$args['shipment']['status_code']}',
                                sub_event_code = '{$args['shipment']['sub_event_code']}',
                                sub_event = '{$args['shipment']['sub_event']}',
                                sub_event_description = '{$args['shipment']['sub_event_description']}'
                                WHERE enviaya_shipment_number = '{$args['shipment']['shipment_number']}'");

                            print_r('{"status":200, "messages":"' . EYHelper::i18n('the_status_was_updated_successfully') . '"}');
                        } catch (Exception $e) {
                            echo EYHelper::i18n('exception'),  $e->getMessage(), "\n";
                        }
                        break;

                    case 'carrier_tracking_number_update':
                        try {

                            $wpdb->get_results("UPDATE {$wpdb->prefix}" . PREFIX . "_shipment SET
                                carrier_shipment_number = '{$args['shipment']['carrier_shipment_number']}'
                                WHERE enviaya_shipment_number = '{$args['shipment']['shipment_number']}'");

                            print_r('{"status":200, "messages":"' . EYHelper::i18n('tracking_number_successfuly_updated') . '"}');
                        } catch (Exception $e) {
                            echo EYHelper::i18n('exception'),  $e->getMessage(), "\n";
                        }
                        break;

                    case 'label_update':
                        try {
                            $wpdb->get_results("UPDATE {$wpdb->prefix}" . PREFIX . "_shipment SET
                                carrier_shipment_number = '{$args['shipment']['carrier_shipment_number']}',
                                label_url = '{$args['shipment']['label']}'
                                WHERE enviaya_shipment_number = '{$args['shipment']['shipment_number']}'");

                            print_r('{"status":200, "messages":"' . EYHelper::i18n('the_label_url_was_updated_successfully') . '"}');
                        } catch (Exception $e) {
                            echo EYHelper::i18n('exception'),  $e->getMessage(), "\n";
                        }
                        break;

                    case 'amounts_update':
                        try {
                            $wpdb->get_results("UPDATE {$wpdb->prefix}" . PREFIX . "_shipment SET
                                carrier_shipment_number = '{$args['shipment']['carrier_shipment_number']}',
                                total_amount = '{$args['shipment']['total_amount']}',
                                net_total_amount = '{$args['shipment']['net_total_amount']}',
                                vat_amount = '{$args['shipment']['vat_amount']}'
                                WHERE enviaya_shipment_number = '{$args['shipment']['shipment_number']}'");

                            print_r('{"status":200, "messages":"' . EYHelper::i18n('the_shipment_amount_was_updated_successfully') . '"}');
                        } catch (Exception $e) {
                            echo EYHelper::i18n('exception'),  $e->getMessage(), "\n";
                        }
                        break;

                    case 'destination_update':
                        try {
                            $states1 = $wpdb->get_results("SELECT order_id FROM {$wpdb->prefix}" . PREFIX . "_shipment
                                WHERE enviaya_shipment_number = '{$args['shipment']['shipment_number']}'");

                            $order_id = $states1[0]->order_id;

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_company']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_company'");


                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_phone']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_phone'");

                            $qry2 = "UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_direction_1']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_address_1'";
                            $states2 = $wpdb->get_results($qry2);

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_direction_2']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_address_2'");

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_country_code']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_country'");

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_postal_code']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_postcode'");

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_city']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_city'");

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_state_code']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_state'");

                            $full_name = $args['shipment']['destination_full_name'];
                            $arr = explode(" ", $full_name);

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$arr[0]}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_first_name'");


                            $arr[0] = '';
                            $full_name = join(' ', $arr);

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$full_name}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_last_name'");

                            $wpdb->get_results("UPDATE {$wpdb->prefix}postmeta SET
                                meta_value = '{$args['shipment']['destination_email']}'
                                WHERE post_id = '{$order_id}' AND meta_key = '_billing_email'");

                            print_r('{"status":200, "messages":"' . EYHelper::i18n('destination_address_successfuly_updated') . '"}');
                        } catch (Exception $e) {
                            echo EYHelper::i18n('exception'),  $e->getMessage(), "\n";
                        }
                        break;

                    case 'origin_update':
                        try {
                            $config1 = get_option('woocommerce_enviaya_settings');
                            $config = json_decode(str_replace('||', '"', $config1['origin_address']));

                            $config->country_code = $args['shipment']['origin_country_code'];
                            $config->state_code = $args['shipment']['origin_state_code'];
                            $config->postal_code = $args['shipment']['origin_postal_code'];
                            $config->city = $args['shipment']['origin_city'];
                            $config->neighborhood = $args['shipment']['origin_neighborhood'];
                            $config->direction_1 = $args['shipment']['origin_direction_1'];
                            $config->direction_2 = $args['shipment']['origin_direction_2'];
                            $config->full_name = $args['shipment']['origin_full_name'];
                            $config->company = $args['shipment']['origin_company'];
                            $config->phone = $args['shipment']['origin_phone'];
                            $config->email = $args['shipment']['origin_email'];

                            $config2 = str_replace('"', '||', json_encode($config));
                            $config1['origin_address'] = $config2;

                            update_option('woocommerce_enviaya_settings', $config1);

                            print_r('{"status":200, "messages":"' . EYHelper::i18n('the_origin_was_updated_successfully') . '"}');
                        } catch (Exception $e) {
                            echo EYHelper::i18n('exception'), $e->getMessage(), "\n";
                        }
                        break;

                    default:
                        print_r('{"status":400, "messages":"' . EYHelper::i18n('an_error_ocurred') . '"}');

                        break;
                }
            } else {
                echo EYHelper::i18n('unsuccessful');
            }
        }
    }

    //dokan
    function prefix_add_enviaya_tab($menu_items)
    {
        $menu_items['enviaya'] = [
            'title' => __('EnvíaYa'),
            'icon' => '<img src="/wp-content/plugins/enviaya-for-woocommerce/public/img/dokan_enviaya_menu.png" style="margin-right: 15px;width: 17px;height: 17px; float:left"></span>',
            'url' => dokan_get_navigation_url('settings/enviaya'),
            'pos' => 200,
            'permission' => 'dokan_view_store_settings_menu',
        ];
        return $menu_items;
    }

    add_filter('dokan_get_dashboard_settings_nav', 'prefix_add_enviaya_tab');


    function prefix_set_enviaya_tab_title($title, $tab)
    {
        if ('enviaya' === $tab) {
            $title = __('EnvíaYa');
        }
        return $title;
    }

    add_filter('dokan_dashboard_settings_heading_title', 'prefix_set_enviaya_tab_title', 10, 2);
    /**
     * Sets the help text for the 'About' settings tab.
     *
     * @param string $help_text
     * @param string $tab
     *
     * @return string Help text for tab with slug $tab
     */
    function prefix_set_enviaya_tab_help_text($help_text, $tab)
    {
        if ('enviaya' === $tab) {
            $help_text = EYHelper::i18n('personalize');
        }
        return $help_text;
    }

    //add_filter('dokan_dashboard_settings_helper_text', 'prefix_set_enviaya_tab_help_text', 10, 2);


    function setup_dokan_form()
    {
        $setup_form = [];

        $setup_form['use_enviaya'] = array(
            'title'       => EYHelper::i18n('use_enviaya'),
            'label'       => ' ',
            'type'        => 'checkbox',
            'name'        => 'dokan_enviaya_use',
            'description' => EYHelper::i18n('use_enviaya_help'),
            'desc_tip'    => true,

        );

        $setup_form['api_key_production'] = array(
            'title'       => EYHelper::i18n('api_key'),
            'type'        => 'text',
            'name'        => 'dokan_enviaya_api_key_production',
            'description' => EYHelper::i18n('api_key_help'),
            'default'     => '',
            'desc_tip'    => true,
        );

        $setup_form['api_key_test'] = array(
            'title'       => EYHelper::i18n('test_api_key'),
            'type'        => 'text',
            'name'        => 'dokan_enviaya_api_key_test',
            'description' => EYHelper::i18n('api_key_help'),
            'default'     => '',
            'desc_tip'    => true,
        );

        $setup_form['timeout'] = array(
            'title'        => EYHelper::i18n('timeout'),
            'type'         => 'text',
            'name'         => 'dokan_enviaya_timeout',
            'description'  => EYHelper::i18n('timeout_help'),
            'default'      => '30',
            'desc_tip'     => true,
        );

        $setup_form['enabled_test_mode'] = array(
            'title'        => EYHelper::i18n('enable_test_mode'),
            'label'        => ' ',
            'default'      => EYHelper::i18n('no'),
            'type'         => 'checkbox',
            'name'         => 'dokan_enviaya_enabled_test_mode',
            'description'  => EYHelper::i18n('test_mode_help'),
            'desc_tip'     => true,
        );

        $setup_form['enviaya_account'] = array(
            'title'       => EYHelper::i18n('account'),
            'type'        => 'select',
            'name'         => 'dokan_enviaya_account',
            'class'    => 'wc-enhanced-select',
            'css'      => 'min-width:300px;',
            'description' => '',
            'default'     => '',
            'desc_tip'    => true,
        );

        $setup_form['sender_address'] = array(
            'title'       => EYHelper::i18n('sender_address'),
            'type'        => 'select',
            'name'         => 'dokan_sender_address',
            'class'    => 'wc-enhanced-select',
            'css'      => 'min-width:300px;',
            'description' => '',
            'default'     => '',
            'desc_tip'    => true,
        );

        return $setup_form;
    }


    function prefix_output_help_tab_content($query_vars)
    {
        if (isset($query_vars['settings']) && 'enviaya' === $query_vars['settings']) {
            if (!current_user_can('dokan_view_store_settings_menu')) {
                dokan_get_template_part('global/dokan-error', '', [
                    'deleted' => false,
                    'message' => EYHelper::i18n('no_access_permissions')
                ]);
            } else {
                $vendor_id = get_current_user_id();
                $url = 'https://' . EYHelper::i18n('api_domain') . '/api/v1/get_accounts?api_key=';
                $value = [];
                foreach (setup_dokan_form() as $item) {
                    $value["{$item['name']}"] = get_user_meta($vendor_id, $item['name'], true);
                }

                $url_sender_address = 'https://' . EYHelper::i18n('api_domain') . '/api/v1/directions?api_key=';

                ?>

                <form method="post" id="settings-form" action="" class="dokan-form-horizontal">
                    <?php wp_nonce_field( 'dokan_enviaya_settings_nonce' ); ?>

                    <input type="hidden" name="enviaya_url" value="<?php echo $url; ?>">
                    <input type="hidden" name="enviaya_url_sender_address" value="<?php echo $url_sender_address; ?>">
                    <?php
                    foreach (setup_dokan_form() as $item) {
                        if ($item['type'] == 'checkbox') {
                            $checked = $value[$item['name']] == 1 ? 'checked' : '';
                            echo "<div class='dokan-form-group'>
                                    <label class='dokan-w3 dokan-control-label' for='{$item['name']}'>{$item['title']}</label>
                                    <div class='dokan-w5 dokan-text-left'>
                                        <div class='checkbox'>
                                            <label>
                                                <input type='hidden' name='{$item['name']}' value='0'>
                                                <input name='{$item['name']}' value='1' id='{$item['name']}' type='{$item['type']}' {$checked}>
                                            </label>
                                        </div>
                                    </div>
                                  </div>";
                        } elseif ($item['type'] == 'select') {
                            echo "<div class='dokan-form-group'>
                                    <label class='dokan-w3 dokan-control-label' for='{$item['name']}'>
                                        {$item['title']}
                                        <span class='dokan-tooltips-help tips' data-placement='bottom' data-original-title='{$item['description']}'>
                                            <i class='fa fa-question-circle'></i>
                                        </span>
                                    </label>
                                    <div class='dokan-w5'>
                                        <select name='{$item['name']}' id='{$item['name']}' class='dokan-form-control'></select>
                                    </div>
                                </div>";
                        } else {
                            echo "<div class='dokan-form-group'>
                                    <label class='dokan-w3 dokan-control-label' for='{$item['name']}'>
                                        {$item['title']}
                                        <span class='dokan-tooltips-help tips' data-placement='bottom' data-original-title='{$item['description']}'>
                                            <i class='fa fa-question-circle'></i>
                                        </span>
                                    </label>
                                    <div class='dokan-w5'>
                                        <input class='dokan-form-control' name='{$item['name']}' value='{$value[$item['name']]}' id='{$item['name']}' type='{$item['type']}'>
                                    </div>
                                </div>";
                        }
                    }
                    ?>

                    <div class="dokan-form-group">
                        <div class="dokan-w4 ajax_prev dokan-text-left" style="margin-left: 25%">
                            <input type="submit" name="dokan_update_enviaya_settings" class="dokan-btn dokan-btn-danger dokan-btn-theme" value="<?php esc_attr_e(EYHelper::i18n('update_settings')); ?>">
                        </div>
                    </div>
                </form>

                

                <script>
                    jQuery(document).ready(function($) {

                        function getApiKey(){
                            var dokan_enviaya_api_key_production = $("input[name='dokan_enviaya_api_key_production']").val();
                            var dokan_enviaya_api_key_test = $("input[name='dokan_enviaya_api_key_test']").val();
                            var api_key = dokan_enviaya_api_key_test ? dokan_enviaya_api_key_test : dokan_enviaya_api_key_production;

                            return api_key;
                        }

                        function enviayaAccounts() {
                            var enviaya_url = $("input[name='enviaya_url']").val();


                            $.ajax({
                                url: enviaya_url + getApiKey(),
                                type: 'GET',
                                success: function(res) {
                                    var mySelect = $("#dokan_enviaya_account");
                                    var selectValues = $.makeArray(JSON.stringify(res['enviaya_accounts']));

                                    $.each(selectValues, function(index, value) {
                                        var account = $.parseJSON(value)[0].account
                                        var alias = $.parseJSON(value)[0].alias

                                        mySelect.empty();
                                        mySelect.append($('<option>', {
                                            value: account,
                                            text: alias
                                        }));
                                    });
                                },
                                error: function() {
                                    var mySelect = $("#dokan_enviaya_account");
                                    mySelect.empty();
                                }
                            });
                        };

                        function ajaxDirection() {
                            var enviaya_url = $("input[name='enviaya_url_sender_address']").val();

                            $.ajax({
                                url: enviaya_url + getApiKey(),
                                success: function (html) {
                                    $('#dokan_sender_address').empty();
                                    var json = '';
                                    html.directions.forEach(function (element) {
                                        element.id = element.id.toString();
                                        var em = new Object();
                                        em.full_name = element.full_name
                                        em.company = element.company
                                        em.phone = element.phone
                                        em.email = element.email
                                        em.direction_1 = element.direction_1
                                        em.direction_2 = element.direction_2
                                        em.neighborhood = element.neighborhood
                                        em.district = element.district
                                        em.postal_code = element.postal_code
                                        em.city = element.city
                                        em.state_code = element.state_code
                                        em.country_code = element.country_code
                                        json = JSON.stringify(em);
                                        json = json.normalize();
                                        json = toASCII(json);
                                        $('#dokan_sender_address').append('<option value=' + btoa(json) + '>' + element.full_name + '</option>');
                                    });
                                },
                                error: function () {
                                    $('#dokan_sender_address').empty();
                                }
                            });
                        }

                        function toASCII(string) {
                            var toFind = ['á', 'é', 'í', 'ó', 'ú', 'ñ', 'Á', 'É', 'Í', 'Ó', 'Ú', 'Ñ'];
                            var toRepl = ['a', 'e', 'i', 'o', 'u', 'n', 'A', 'E', 'I', 'O', 'U', 'N'];

                            for (var i = 0; i < toFind.length; i++) {
                                var arr = string.split(toFind[i]);
                                string = arr.join(toRepl[i]);
                            }

                            var arr = string.split('null');
                            string = arr.join('\"\"');
                            return string;
                        }

                        enviayaAccounts();
                        ajaxDirection();
                        $("input").keyup(function() {
                            enviayaAccounts();
                            ajaxDirection();
                        });
                    });


                </script>

                <style>
                    #settings-form p.help-block {
                        margin-bottom: 0;
                    }
                </style>
                <?php
            }
        }
    }

    add_action('dokan_render_settings_content', 'prefix_output_help_tab_content');

    /*
    function prefix_save_enviaya_settings($user_id)
    {
        // Bail if another settings tab is being saved
        foreach (setup_dokan_form() as $item) {
            update_user_meta($user_id, $item['name'], sanitize_text_field($_POST["{$item['name']}"]));
        }
    }

    add_action('dokan_store_profile_saved', 'prefix_save_enviaya_settings');
    */

    function prefix_save_about_settings() {
      $vendor_id   = dokan_get_current_user_id();
      $post_data = wp_unslash( $_POST );
      $nonce     = isset( $post_data['_wpnonce'] ) ? $post_data['_wpnonce'] : '';

      // Bail if another settings tab is being saved
      if ( ! wp_verify_nonce( $nonce, 'dokan_enviaya_settings_nonce' ) ) {
          return;
      }

      foreach (setup_dokan_form() as $item) {

        update_user_meta($vendor_id, $item['name'], sanitize_text_field($post_data["{$item['name']}"]));

      }

      wp_send_json_success( array(
        'msg' => __( 'Your information has been saved successfully' ),
      ) );    

    }

    //Hooked with priority 5 to run before WeDevs\Dokan\Dashboard\Templates::ajax_settings()
    add_action( 'wp_ajax_dokan_settings', 'prefix_save_about_settings', 5 );

    add_filter('woocommerce_billing_fields', 'remove_enviaya_billing_phone_field', 20);
    function remove_enviaya_billing_phone_field($fields)
    {
        $fields['billing_phone']['required'] = true; // To be sure "NOT required"

        return $fields;
    }


    add_action('rest_api_init', function () {
        register_rest_route('enviaya/v1', '/webhook', array(
            'methods' => 'POST',
            'callback' => 'my_enviaya_func',
            'permission_callback' => '__return_true'
        ));
    });

    add_filter('woocommerce_cart_shipping_method_full_label', 'enviaya_shipping_service_design_advanced_label', 10, 2);
    function enviaya_shipping_service_design_advanced_label($label, $method)
    {

        $meta_data = $method->get_meta_data();
        $shipping_carrier_name = get_option('woocommerce_enviaya_settings')['shipping_carrier_name'];

        if (isset($meta_data['carrier_name'])) {
            if ($shipping_carrier_name === "1") {
                $label = $meta_data['carrier_name'] . " - " . $label;
            }
        }

        return $label;
    }

    add_filter( 'woocommerce_cart_totals_before_shipping', function() {
        if(is_cart())
        {
            $settings = EyHelper::settings();
            if(!empty($settings['title_rating_configuration']))
            {
                echo '<label>'.$settings['title_rating_configuration'].'</label>';
            }


        }//end
    });

    function set_message_checkout() {
        if(is_checkout())
        {
            //
            if ( ! defined( 'ABSPATH' ) ) {
                exit;
            }//end

            //only run normal pages,
            if ( ! is_ajax() ) {
                $settings = EyHelper::settings();
                if(!empty($settings['title_rating_configuration']))
                {
                    echo '<label>'.$settings['title_rating_configuration'].'</label>';
                }//end
            }
        }//end
    }//end
    add_filter( 'woocommerce_review_order_after_cart_contents', "set_message_checkout");

    //Add functionality for check if order is pay
    add_action('woocommerce_order_status_completed','insert_points', 10, 1);

    function insert_points($order_id)
    {
        global $wpdb;
        
        $settings = EYHelper::settings();   
        $post_data = ( !empty($_POST) ) ? $_POST :  $_GET;
        
        
        if($settings["automatic_shipment_booking"] === "yes")
        {

            if($post_data['action'] == 'editpost'){

                $props = [
                    'get_post'              => get_post($order_id),
                    'wc_get_order'          => wc_get_order($order_id),
                ];
                
                $order = wc_get_order($order_id);
        
                if( $order->has_shipping_method( 'enviaya' ) ) {

                    $shipping = reset( $order->get_shipping_methods() );
                    
                    $props['rate_id'] = $shipping->get_meta( 'rate_id' );
                    $props['shipment_id'] = $shipping->get_meta( 'eya_shipment_id' );
 
                }

                //create shipment
                $params_request = EYHelper::create_shipment($props);
                
                $result    = EYHelper::libAPI()->create($params_request);
                $response2 = $result['response'];
                $request2 = $result['request'];
                
                error_log("NEW ORDER REQUEST: " . json_encode($result['request']));
                logAPI(json_encode($result['request']), 'shipment_request');
                
                error_log("NEW ORDER RESPONSE: " . json_encode($response2));
                logAPI(json_encode($response2), 'shipment_response');                

                if (
                    isset($response2->enviaya_shipment_number)
                    && isset($response2->carrier_shipment_number)
                ) {

                    update_post_meta($order_id, "eya_is_auto_shipment_booking", "yes");
                    update_post_meta($order_id, "eya_status_shipment_booking", "done");

                    $rate =  $wpdb->get_row( $wpdb->prepare( "SELECT * FROM  {$wpdb->prefix}enviaya_rates WHERE order_id=%s AND rate_id=%s", $order_id, $request2->rate_id ) ); 


                    $wpdb->query("INSERT INTO {$wpdb->prefix}" . PREFIX . "_shipment (
                        rate_id, order_id, carrier, carrier_logo_url, estimated_delivery, carrier_service_name,
                        carrier_service_code, total_amount, net_total_amount, currency, enviaya_shipment_number,
                        carrier_shipment_number, label_url, shipment_status, webhook_status
                        ) VALUES (
                            '{$request2->rate_id}', '{$order_id}', '{$response2->carrier}', '{$response2->rate->carrier_logo_url}',
                            '{$rate->estimated_delivery}', '{$rate->carrier_service_name}',
                            '{$response2->carrier_service_code}', '{$rate->total_amount}',
                            '{$rate->net_total_amount}', '{$response2->rate->currency}',
                            '{$response2->enviaya_shipment_number}', '{$response2->carrier_shipment_number}',
                            '{$response2->label_share_link}', 'Waiting', 'empty' )");
                }
  

            }else{

                $flag_auto   = get_post_meta($order_id, "eya_is_auto_shipment_booking", true );
                $status_auto = get_post_meta($order_id, "eya_status_shipment_booking", true );

                if($status_auto == "done" || $status_auto == "fail")
                {
                    return;
                }          
                
                update_post_meta($order_id, "eya_message_auto_shipment", EYHelper::i18n('automatic_booking_message_on_await'));            
                update_post_meta($order_id, "eya_is_auto_shipment_booking", "yes");
                update_post_meta($order_id, "eya_status_shipment_booking", "await"); // await, fail, done
                
                return false;

            }

            
        }        
    }//end function order complete
    
    /**
        * Handle a custom 'customvar' query var to get orders with the 'customvar' meta.
        * @param array $query - Args for WP_Query.
        * @param array $query_vars - Query vars from WC_Order_Query.
        * @return array modified $query
        */
    function eya_handle_custom_query_var( $query, $query_vars ) {
        if ( ! empty( $query_vars['is_auto_shipment_booking'] ) ) {
            $query['meta_query'][] = array(
                    'key'   => 'eya_is_auto_shipment_booking',
                    'value' => esc_attr( $query_vars['is_auto_shipment_booking'] ),
            );
        }
        
        if ( ! empty( $query_vars['status_shipment_booking'] ) ) {
            $query['meta_query'][] = array(
                    'key'   => 'eya_status_shipment_booking',
                    'value' => esc_attr( $query_vars['status_shipment_booking'] ),
            );
        }     
        
        return $query;
    }
    add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', 'eya_handle_custom_query_var', 10, 2 );
    
   
    function enviaya_start_scheduler_cronjob()
    {
        $settings = EYHelper::settings();
        
        if ( false === as_next_scheduled_action( 'eg_midnight_log' ) )
        {
            as_schedule_recurring_action( strtotime( 'now' ), 30, 'eg_midnight_log' );
	    }
    }
    add_action("enviaya_enable_scheduler", "enviaya_start_scheduler_cronjob");
    
    /**
    * A callback to run when the 'eg_midnight_log' scheduled action is run.
    */
    function eg_log_action_data() 
    {
       global $wpdb;
        
       $settings = EYHelper::settings();
       $args     = array( 
           'status'                   => array('wc-completed'),
           'is_auto_shipment_booking' => 'yes',
           'status_shipment_booking'  => 'await',
           'limit' => 3, 
           'offset' => 0,
       );
       $orders = wc_get_orders( $args );     
       
       error_log( 'It is just after one minute on ' . date( 'Y-m-d h:i:s' )." For :: ".sizeof($orders)." Orders" );
       
       foreach($orders as $order)
       {
           
           if (isset($settings['origin_address']->country_code) && isset($settings['origin_address']->postal_code)) 
            {
                $order_id = $order->get_id();
                
                $props = [
                    'get_post'              => get_post($order->get_id()),
                    'wc_get_order'          => wc_get_order($order->get_id()),
                ];
                
                //Snippet for build shipment
                $params_request = EYHelper::create_shipment($props);
                
                if(!$params_request)
                {                               
                    return;
                }
                   
                $result    = EYHelper::libAPI()->create($params_request);
                $response2 = $result['response'];
                
                error_log("NEW ORDER 1 REQUEST: " . json_encode($result['request']));
                logAPI(json_encode($result['request']), 'shipment_request');
                
                error_log("NEW ORDER 1 RESPONSE: " . json_encode($response2));
                logAPI(json_encode($response2), 'shipment_response');                

                //asking for secondary service
                if( ( isset($response2->errors) || is_null($response2) ) && $settings["eya_setting_secondary_service"] == "secondary" )
                {
                    //Snippet for build shipment
                    $params_request["carrier"]              = $settings["eya_new_option_secondary_carrier"];
                    $params_request["carrier_service_code"] = $settings["eya_new_option_secondary_service"];

                    $result    = EYHelper::libAPI()->create($params_request);
                    $response2 = $result['response'];

                    error_log("NEW ORDER 1 REQUEST SECONDARY: " . json_encode($result['request']));
                    logAPI(json_encode($result['request']), 'shipment_request');
                    
                    error_log("NEW ORDER 1 RESPONSE SECONDARY: " . json_encode($response2));
                    logAPI(json_encode($response2), 'shipment_response');
                }

                if (isset($response2->enviaya_shipment_number) && isset($response2->carrier_shipment_number)) 
                {
                    $wpdb->query("INSERT INTO {$wpdb->prefix}" . PREFIX . "_shipment (
                        rate_id, order_id, carrier, carrier_logo_url, estimated_delivery, carrier_service_name,
                        carrier_service_code, total_amount, net_total_amount, currency, enviaya_shipment_number,
                        carrier_shipment_number, label_url, shipment_status, webhook_status
                        ) VALUES (
                            '{$response2->rate->rate_id}', '{$order_id}', '{$response2->rate->carrier}', '{$response2->rate->carrier_logo_url}',
                            '{$response2->rate->estimated_delivery}', '{$response2->rate->carrier_service_name}',
                            '{$response2->rate->carrier_service_code}', '{$response2->rate->total_amount}',
                            '{$response2->rate->net_total_amount}', '{$response2->rate->currency}',
                            '{$response2->enviaya_shipment_number}', '{$response2->carrier_shipment_number}',
                            '{$response2->label_share_link}', 'Waiting', 'empty' )");

                    $success_message_existing = str_replace(
                        ['{#carrier_tracking_number#}', '{#carrier#}', '{#service_name#}', '{#label_share_link#}'],
                        [ $response2->carrier_shipment_number, $response2->rate->carrier, $response2->rate->carrier_service_name, $response2->label_share_link],
                        EYHelper::i18n('automatic_booking_success_message')
                    );

                    //news datas
                    update_post_meta( $order_id, "eya_carrier", $response2->carrier );
                    update_post_meta( $order_id, "eya_carrier_service_name", $response2->rate->carrier_service_name );
                    update_post_meta( $order_id, "eya_carrier_service_code", $response2->rate->carrier_service_code );
                    update_post_meta( $order_id, "eya_rate_id", $response2->rate->rate_id );
                    update_post_meta( $order_id, "eya_shipment_id", $response2->rate->shipment_id );
                    update_post_meta( $order_id, "eya_message_auto_shipment", $success_message_existing );
                    update_post_meta( $order_id, "eya_status_shipment_booking", "done" );

                    return;
                }
                
                if( isset($response2->errors) )
                {
                    // If you don't have the WC_Order object (from a dynamic $order_id)
                    $order = wc_get_order( $order_id );
                    $order->add_order_note( __($response2->errors) );
                }

                update_post_meta($order_id, "eya_message_auto_shipment", EYHelper::i18n('automatic_booking_error_message_default'));
                update_post_meta($order_id, "eya_status_shipment_booking", "fail");
                
                return;
            }//end if*/
            
       }//end foreach
    }//end hook eg_log_action_data
    add_action( 'eg_midnight_log', 'eg_log_action_data' );    
    
    
    function enviaya_end_scheduler_cronjob()
    {
        $settings = EYHelper::settings();
        
        as_unschedule_all_actions( "eg_midnight_log" );
    }
    add_action("enviaya_disable_scheduler", "enviaya_end_scheduler_cronjob");
    
    function admin_notice_order()
    {
        global $pagenow;        
        $pages    = ["post.php"];
        $settings = EYHelper::settings();
        
        if($settings["automatic_shipment_booking"] === "yes")
        {  
            if(in_array( $pagenow, $pages ))
            {
                $order        = get_post();
                $message      = "";      
                $notice_class = "notice-info"; 
                // notice-error; notice-success; notice-info; notice-warning  

                if($order)
                {   
                    $data_order    = wc_get_order($order->ID);
                    if($data_order){
                        $message       = get_post_meta($data_order->get_id(), "eya_message_auto_shipment", true);
                        $status_order  = get_post_meta($data_order->get_id(), "eya_status_shipment_booking", true);
                        if(!empty($message))
                        {
                            if($status_order == "fail")
                            {
                                $notice_class = "notice-warning";
                            }

                            echo '<div class="notice '.$notice_class.' is-dismissible"><p>'.$message.'</p></div>';
                            
                            return;
                        }
                    }
                }
            }

            if($pagenow === "edit.php")
            {
                $args = array( 
                    'status'                   => array('wc-completed'),
                    'is_auto_shipment_booking' => 'yes',
                    'status_shipment_booking'  => 'await'                   
                );
                $orders = wc_get_orders( $args );
                $count_orders = sizeof($orders);

                $error_message_bulk_action = str_replace(
                    '{#count_orders#}', 
                    $count_orders,
                    EYHelper::i18n('automatic_booking_message_bulk_action')
                );

                if($count_orders > 0)
                    echo '<div class="updated fade" ><p>'.$error_message_bulk_action.'</p></div>';
                
                return;
            }
        }
    }
    add_action('admin_notices', 'admin_notice_order' , 10, 0 );      
    
    add_filter("update_footer", "footer_html");
    function footer_html()
    {
        global $pagenow;
        
        if($pagenow == "edit.php"){
            add_thickbox();

            $settings = EYHelper::settings();
            $carriers = EYHelper::getOptionsAvailableCarriers($settings['eya_new_option_carrier']);
            $services = EYHelper::getOptionsAvailableServices($settings['eya_new_option_carrier'], $settings['eya_new_option_service']);
            $carriers_secondary = EYHelper::getOptionsAvailableCarriers($settings['eya_new_option_secondary_carrier']);
            $services_secondary = EYHelper::getOptionsAvailableServices($settings['eya_new_option_secondary_carrier'], $settings['eya_new_option_secondary_service']);

            echo '<div id="modal-admin-id" style="display:none;">                    
                    <h3>'.EYHelper::i18n('global_settings_service_type_field').'</h3>
                    <table class="form-table">
                        <tr>
                            <td>
                                <select class="wc-enhanced-select" id="select-option-service" style="width: 100% !important;">
                                    <option value="default" '.($settings['eya_option_service'] == "default" ? 'selected' : '' ).'  >'.EYHelper::i18n('global_settings_option_service_type_default').'</option>
                                    <option value="other" '.($settings['eya_option_service'] == "other" ? 'selected' : '' ).'>'.EYHelper::i18n('global_settings_option_service_type_other').'</option>                                    
                                </select>
                            </td>
                        </tr>
                    </table>
                    <table class="form-table">
                        <tr class="'.($settings['eya_option_service'] == "other" ? '' : 'eya-hidden-elements-form' ).'" data-settings-form="main-service">
                            <td class="select-options-carriers">
                                <label for="new-option-carrier" style="margin-bottom:10px; font-weight: bold; display: inline-block;">'.EYHelper::i18n('global_settings_carrier_field').'</label>
                                <select class="wc-enhanced-select" id="new-option-carrier" data-eya-carrier="new-option-service" style="width: 100% !important;">
                                    '.$carriers.'
                                </select>
                            </td>
                            <td class="select-options-services">
                                <label for="new-option-service" style="margin-bottom:10px; font-weight: bold; display: inline-block;">'.EYHelper::i18n('global_settings_service_field').'</label>
                                <select class="wc-enhanced-select" id="new-option-service" style="width: 100% !important;">
                                    '.$services.'
                                </select>
                            </td>
                        <tr>
                    </table>
                    <h3>'.EYHelper::i18n('global_settings_heading') . wc_help_tip(EYHelper::i18n('global_settings_checkbox_tooltip')).'</h3>
                    <table class="form-table">    
                        <tr>
                            <td>
                                <input type="checkbox" name="setting-secondary-service" value="secondary" '.($settings['eya_setting_secondary_service'] == 'secondary' ? 'checked' : '').' id="setting-secondary-service" />
                            </td>
                            <td> 
                                <label for="setting-secondary-service">'.EYHelper::i18n('global_settings_checkbox_field') .'</label>
                            </td>
                        <tr>                        
                    </table>
                    <table class="form-table">
                        <tr class="'.((string)$settings['eya_setting_secondary_service'] === "secondary" ? "" : "eya-hidden-elements-form").'" data-settings-form="secondary-service">
                            <td class="select-options-carriers">
                                <label for="new-option-secondary-carrier" style="margin-bottom:10px; font-weight: bold; display: inline-block;">'.EYHelper::i18n('global_settings_carrier_field').'</label>
                                <select class="wc-enhanced-select" id="new-option-secondary-carrier" data-eya-carrier="new-option-secondary-service" style="width: 100% !important;">
                                    '.$carriers_secondary.'
                                </select>
                            </td>
                            <td class="select-options-services">
                                <label for="new-option-secondary-service" style="margin-bottom:10px; font-weight: bold; display: inline-block;">'.EYHelper::i18n('global_settings_service_field').'</label>
                                <select class="wc-enhanced-select" id="new-option-secondary-service" style="width: 100% !important;">
                                    '.$services_secondary.'
                                </select>
                            </td>
                        <tr>
                    </table>
                    <table class="form-table">
                        <tr>
                            <td colspan="2">
                                <button id="eya-do-setting" class="button-primary">'.EYHelper::i18n('global_settings_button_field').'</button>                                
                                <a href="#" data-close style="margin-left:5px">'.EYHelper::i18n('global_settings_button_close_field').'</a>
                            </td>
                        </tr>                       
                    </table>        
                </div>
                <a href="#TB_inline?width=600&height=550&inlineId=modal-admin-id" style="display: none;" name="'.EYHelper::i18n('global_settings_bulk_shipments_title').'" id="thickbox-admin" class="thickbox">click here</a>';
        }
    }

    // EY CRON SCHEDULES
    add_filter( 'cron_schedules', 'add_ey_cron_schedules' );
    
    function add_ey_cron_schedules( $schedules ) {

        $schedules['ey_cs_daily'] = array(
            'interval' => 86400,
            'display' => EYHelper::i18n('cron_schedule_daily')
        );

        $schedules['ey_cs_weekly'] = array(
            'interval' => 604800,
            'display' => EYHelper::i18n('cron_schedule_weekly')
        );

        $schedules['ey_cs_twiceweekly'] = array(
            'interval' => 302400,
            'display' => EYHelper::i18n('cron_schedule_twiceweekly')
        );

        $schedules['ey_cs_monthly'] = array(
            'interval' => 2592000,
            'display' => EYHelper::i18n('cron_schedule_monthly')
        );


        return $schedules;

    }


    //Cron events
    
    add_action('eycs_logs_cleaner', 'do_eycs_logs_cleaner');
 
    function do_eycs_logs_cleaner() {
        
        $logger = wc_get_logger();
        $logger->clear('enviaya-rating-logger');
    }

    add_action('eycs_rates_cleaner', 'do_eycs_rates_cleaner');
 
    function do_eycs_rates_cleaner() {
        
        global $wpdb;

        $wpdb->query( $wpdb->prepare( "DELETE FROM  {$wpdb->prefix}enviaya_rates" ) ); 

        
        
    }

    
    
    
} else {

    require_once __DIR__ . "/admin/helpers/EYHelper.php";

    printf('<div class="error"><p>' . EYHelper::getReplacingKeyi18n('{#brand_name#}', 'brand_name', 'woocommerce_disabled_message') . '</p></div>');
    deactivate_plugins('enviaya-for-woocommerce/enviaya-for-woocommerce.php');
}

/**
 * Write log into file if logging is enabled
 * @param string Message which will be written into the file
 * @param string type of log, different types writes into different files
 *
 * @return bool
 */

function logAPI ($message, $type){
    if(get_option('woocommerce_enviaya_settings')['enable_api_logging'] != 'yes'){
        return null;
    }

    $logger = wc_get_logger();
    $logger->info( $type. " => ". $message, array( 'source' => 'enviaya-rating-logger' ) );

    return true;
}

function log_it( $message ) {
   if( WP_DEBUG === true ){
     if( is_array( $message ) || is_object( $message ) ){
       //error_log will be located according to server configuration
       //you can specify a custom location if needed like this
       //error_log( $var, 0, "full-path-to/error_log.txt")
       error_log( print_r( $message, true ) );
     } else {
       error_log( $message );
     }
   }
}